/*
Tecnisa - Tecnologia em Informtica

Nome: Imposto

Descrio:
Procedimento para calcular os Impostos dos produtos.

Criao:
30/06/03 - Andr Rodigo


@C:\WIN_V2\SQL\END_CONV\ORACLE\8695_VMNSTP_IMPOSTO.SQL

*/


CREATE OR REPLACE PROCEDURE IMPOSTO_VMN(
   P_CDFILIAL     IN IMPITVENDAVM.CDFILIAL%TYPE,
   P_CDPRODUTO    IN IMPITVENDAVM.CDPRODUTO%TYPE,
   P_DTIMPITVENDA IN IMPITVENDAVM.DTIMPITVENDA%TYPE,
   P_EMPRESA      IN ALIQUOTA.CDEMPRESA%TYPE,
   P_INSCESTA     IN ALIQUOTA.CDINSCESTA%TYPE,
   P_FILDEST      IN FILIAL.CDFILIAL%TYPE,
   P_CLIENTE      IN CLIENTE.CDCLIENTE%TYPE,
   P_VRPRODUTO    IN ITEMAPABASEV.VRPRITEMMAP%TYPE,
   P_INIVALID     IN ALIQUOTA.DTINIVALID%TYPE,
   P_VRIMPOSTO    OUT IMPITVENDAVM.VRIMPITVENDA%TYPE)

IS
   V_CDGRUPOFISC  VARCHAR2(3);

   V_INSCESTA     ALIQUOTA.CDINSCESTA%TYPE;
   V_CDARVPROD    PRODUTO.CDARVPROD%TYPE;
   V_PAISORIG     VARCHAR2(04);
   V_ESTAORIG     VARCHAR2(02);

   V_PAISDEST     VARCHAR2(04);
   V_ESTADEST     VARCHAR2(02);
   V_MUNIDEST     VARCHAR2(04);

   V_CARACFIS     VARCHAR2(01);
   V_INCITEMP     VARCHAR2(18);
   V_ALIQTEMP     VARCHAR2(18);
   V_CFOPTEMP     VARCHAR2(18);
   V_CSTTEMP      VARCHAR2(18);
   V_FISOPEITEM   VARCHAR2(11);
   V_SITUTRIBPR   VARCHAR2(03);
   V_IDTRATIMPBC   VARCHAR2(1);
   V_IDIMPORECUPBC VARCHAR2(1);

   V_LIQUIITEM    NUMBER;
   V_PARDEDTMP    NUMBER;
   V_PREDBCTMP    NUMBER;
   V_ALIQUTMP     NUMBER;
   V_IMPOSTOIT    NUMBER;
   V_TAXABASECALC NUMBER;
   V_TAXAALIQUOTA NUMBER;
   V_BASECALCIT   NUMBER;
   V_CONTABCALC   NUMBER;
   V_PERDBCALC    NUMBER;
   V_ADIMPAF      NUMBER;
   V_NRIMPDIGI    NUMBER;
   V_VRIMPITAFAUX NUMBER;
   V_TOTAIMPBCALC NUMBER;

BEGIN
  CONSULTA_GRUPOFISCAL('C', P_CLIENTE, '', 'S', '01', V_CDGRUPOFISC);

  SELECT MAX(CDINSCESTA) INTO V_INSCESTA
  FROM FILIAL
  WHERE CDFILIAL  = P_FILDEST
    AND CDEMPRESA = P_EMPRESA;

   SELECT MAX(CDPAIS), MAX(SGESTADO) INTO V_PAISORIG, V_ESTAORIG
   FROM INSCRESTAD
   WHERE (CDEMPRESA  = P_EMPRESA)
     AND (CDINSCESTA = V_INSCESTA);

   SELECT MAX(CDPAIS), MAX(SGESTADO), MAX(CDMUNICIPIO) INTO V_PAISDEST, V_ESTADEST, V_MUNIDEST
   FROM ENDECLIE
   WHERE (CDCLIENTE    = P_CLIENTE) AND
         (IDTPENDECLIE = 'P');

   SELECT MAX(IDCARACFIS) INTO V_CARACFIS
   FROM PARAMFILIAL
   WHERE CDFILIAL = P_FILDEST;

   SELECT MAX(CDARVPROD) INTO V_CDARVPROD
   FROM PRODUTO
   WHERE CDPRODUTO = P_CDPRODUTO;

   DELETE FROM IMPITVENDAVM
   WHERE CDFILIAL     = P_CDFILIAL
     AND CDPRODUTO    = P_CDPRODUTO
     AND DTIMPITVENDA = P_DTIMPITVENDA;

   V_INCITEMP  := P_CDFILIAL||P_CDPRODUTO;
   V_LIQUIITEM := P_VRPRODUTO;

   CONSULTA_INCIDENCIA('S', P_EMPRESA, V_CDARVPROD, V_INSCESTA, '01',
                       P_CDFILIAL, P_CDPRODUTO, V_CDGRUPOFISC);

   V_ALIQTEMP := V_INCITEMP;
   FOR C_INCIDENCIA IN
     (SELECT DISTINCT (CDIMPOTEMP) AS CDIMPOTEMP, CDFISCTEMP,
             IP.IDTRATIMPO, IP.IDIMPORECUP
      FROM INCITEMP IT, IMPOSTO IP
      WHERE (IT.NRINCITEMP = V_INCITEMP)    AND
            (IP.CDIMPOSTO  = IT.CDIMPOTEMP) AND
            (IP.IDPRODNOTA = 'P')) LOOP

      CONSULTA_ALIQUOTA(C_INCIDENCIA.CDIMPOTEMP, 'S', P_EMPRESA, V_CDARVPROD,
                        V_INSCESTA, V_PAISORIG, V_ESTAORIG, V_PAISDEST, V_ESTADEST,
                        V_MUNIDEST, V_LIQUIITEM, P_INIVALID, P_CDFILIAL, P_CDPRODUTO,
                        V_CDGRUPOFISC);

      V_PARDEDTMP  := 0;
      V_PREDBCTMP  := 0;
      V_ALIQUTMP   := 0;
      V_IMPOSTOIT  := 0;

      SELECT MAX(VRPARDEDTMP), MAX(VRPREDBCTMP), MAX(VRALIQUTMP)
      INTO V_PARDEDTMP, V_PREDBCTMP, V_ALIQUTMP
      FROM ALIQTEMP
      WHERE (NRALIQTEMP = V_ALIQTEMP);

      IF (V_ALIQUTMP IS NULL) THEN
         V_ALIQUTMP := 0;
      END IF;

      IF (V_PREDBCTMP IS NULL) THEN
         V_PREDBCTMP := 0;
      END IF;

      IF (V_PARDEDTMP IS NULL) THEN
          V_PARDEDTMP := 0;
      END IF;

      V_TAXABASECALC := (100 - V_PREDBCTMP)/100;
      V_TAXAALIQUOTA := (V_ALIQUTMP/100);
      V_BASECALCIT   := V_LIQUIITEM * V_TAXABASECALC;
      V_IMPOSTOIT    := ((V_LIQUIITEM * V_TAXABASECALC) * V_TAXAALIQUOTA) - V_PARDEDTMP;

      V_CONTABCALC := V_LIQUIITEM - (V_LIQUIITEM * V_TAXABASECALC);
      V_PERDBCALC  := 100 - V_PREDBCTMP;

      IF (V_IMPOSTOIT < 0) THEN
         V_IMPOSTOIT := 0;
      END IF;

      V_ADIMPAF := 0;

      IF (C_INCIDENCIA.IDTRATIMPO = 'A') THEN
         V_ADIMPAF := V_IMPOSTOIT;

      ELSIF (C_INCIDENCIA.IDTRATIMPO = 'D') THEN
         V_ADIMPAF := - V_IMPOSTOIT;

      END IF;

      SELECT COUNT(*) INTO V_NRIMPDIGI
      FROM IMPITVENDAVM
      WHERE (CDFILIAL     = P_CDFILIAL)     AND
            (CDPRODUTO    = P_CDPRODUTO)    AND
            (DTIMPITVENDA = P_DTIMPITVENDA) AND
            (CDIMPOSTO   = C_INCIDENCIA.CDIMPOTEMP);

      IF V_NRIMPDIGI = 0 THEN
         INSERT INTO IMPITVENDAVM(
            CDFILIAL, CDPRODUTO, DTIMPITVENDA,
            CDIMPOSTO, VRIMPITVENDA, VRPERREDBCVE, VRBASECALCVE,
            VRALIQITMVE, VRPARCDEDVE, CDFISCITMVE, VRADIMPVE)
         VALUES(
            P_CDFILIAL, P_CDPRODUTO, P_DTIMPITVENDA,
            C_INCIDENCIA.CDIMPOTEMP, V_IMPOSTOIT, V_PREDBCTMP, V_BASECALCIT, V_ALIQUTMP,
            V_PARDEDTMP, C_INCIDENCIA.CDFISCTEMP, V_ADIMPAF);

      ELSE
         SELECT MAX(VRIMPITVENDA) INTO V_VRIMPITAFAUX
         FROM IMPITVENDAVM
         WHERE (CDFILIAL     = P_CDFILIAL)     AND
               (CDPRODUTO    = P_CDPRODUTO)    AND
               (DTIMPITVENDA = P_DTIMPITVENDA) AND
               (CDIMPOSTO   = C_INCIDENCIA.CDIMPOTEMP);

         IF (C_INCIDENCIA.IDTRATIMPO = 'D') THEN
            V_VRIMPITAFAUX := -V_VRIMPITAFAUX;
         END IF;
         IF (C_INCIDENCIA.IDTRATIMPO <> 'V') THEN
            UPDATE IMPITVENDAVM
            SET VRADIMPVE = V_VRIMPITAFAUX
            WHERE (CDFILIAL     = P_CDFILIAL)     AND
                  (CDPRODUTO    = P_CDPRODUTO)    AND
                  (DTIMPITVENDA = P_DTIMPITVENDA) AND
                  (CDIMPOSTO   = C_INCIDENCIA.CDIMPOTEMP);
         END IF;
      END IF;

      DELETE ALIQTEMP
      WHERE NRALIQTEMP = V_ALIQTEMP;

      DELETE INCITEMP
      WHERE NRINCITEMP = V_INCITEMP;
   END LOOP;

---------------------------------------------------------------------------------------------------------
--Clculo do Valor dos Impostos que contm outros impostos na sua Base de Clculo

   V_CFOPTEMP := V_INCITEMP;
   FOR C_PRODUTO IN (SELECT II.CDPRODUTO, II.CDIMPOSTO, VRPERREDBCVE,
                            VRALIQITMVE, VRPARCDEDVE, CDFISCITMVE, VRIMPITVENDA, IDTRATIMPO,
                            IM.IDIMPORECUP
                     FROM IMPITVENDAVM II, IMPOSTO IM
                     WHERE (II.CDFILIAL     = P_CDFILIAL)     AND
                           (II.CDPRODUTO    = P_CDPRODUTO)    AND
                           (II.DTIMPITVENDA = P_DTIMPITVENDA) AND
                           (IM.CDIMPOSTO    = II.CDIMPOSTO))
   LOOP
      CONSULTA_CST('S', P_EMPRESA, C_PRODUTO.CDPRODUTO, P_INSCESTA, '01', P_CDFILIAL, P_CDPRODUTO, V_CDGRUPOFISC);

      CONSULTA_CFOP('S', P_EMPRESA, C_PRODUTO.CDPRODUTO, P_INSCESTA, '01', V_PAISORIG,
                    V_ESTAORIG, V_PAISDEST, V_ESTADEST, P_CDFILIAL, P_CDPRODUTO,
                    V_CDGRUPOFISC, P_DTIMPITVENDA);

      SELECT MAX(CDFISOPETMP) INTO V_FISOPEITEM
      FROM CFOPTEMP
      WHERE NRCFOPTEMP = V_CFOPTEMP;

      SELECT MAX(CDSITTRIBTMP) INTO V_SITUTRIBPR
      FROM CSTTEMP
      WHERE NRCSTTEMP = V_CSTTEMP;

      SELECT SUM(VRIMPITVENDA) AS VRIMPOBCALC, MAX(IMPOSTO.IDTRATIMPO), MAX(IDIMPORECUP)
      INTO V_TOTAIMPBCALC, V_IDTRATIMPBC, V_IDIMPORECUPBC
      FROM IMPITVENDAVM II, BASECALC BC, IMPOSTO
      WHERE (BC.CDIMPOSTO      = C_PRODUTO.CDIMPOSTO) AND
            (II.CDFILIAL       = P_CDFILIAL)          AND
            (II.CDPRODUTO      = P_CDPRODUTO)           AND
            (II.DTIMPITVENDA   = P_DTIMPITVENDA)      AND
            (II.CDPRODUTO      = C_PRODUTO.CDPRODUTO) AND
            (II.CDIMPOSTO      = BC.CDIMPOBCALC)      AND
            (IMPOSTO.CDIMPOSTO = BC.CDIMPOBCALC)      AND
            ((V_SITUTRIBPR = BC.CDCSTIMPO) OR (BC.CDCSTIMPO IS NULL)) AND
            ((SUBSTR(V_FISOPEITEM,2,10) = BC.CDCFOPIMPO) OR (BC.CDCFOPIMPO IS NULL)) AND
            ((II.CDFISCITMVE = BC.CDFISCIMPO) OR (BC.CDFISCIMPO IS NULL)) AND
            (BC.IDTIPOBCALC  = 'I');

      DELETE FROM CFOPTEMP
      WHERE NRCFOPTEMP = V_CFOPTEMP;

      DELETE FROM CSTTEMP
      WHERE NRCSTTEMP = V_CSTTEMP;

      IF NOT (V_TOTAIMPBCALC IS NULL) THEN
         V_TAXABASECALC := (100 - C_PRODUTO.VRPERREDBCVE)/100;
         V_TAXAALIQUOTA := (C_PRODUTO.VRALIQITMVE/100);

         V_ADIMPAF := 0;

         IF (V_IDTRATIMPBC = 'A') THEN
            V_ADIMPAF := ((V_TOTAIMPBCALC * V_TAXABASECALC) * V_TAXAALIQUOTA);

         ELSIF (V_IDTRATIMPBC = 'D') THEN
            V_ADIMPAF := - ((V_TOTAIMPBCALC * V_TAXABASECALC) * V_TAXAALIQUOTA);

         ELSIF (V_IDTRATIMPBC  = 'R') THEN
            IF (V_IDIMPORECUPBC = 'S') THEN
               IF (C_PRODUTO.CDFISCITMVE <> '1') THEN
                  V_ADIMPAF := - ((V_TOTAIMPBCALC * V_TAXABASECALC) * V_TAXAALIQUOTA);
               END IF;
            ELSE
               V_ADIMPAF := - ((V_TOTAIMPBCALC * V_TAXABASECALC) * V_TAXAALIQUOTA);
            END IF;
         END IF;

         IF (C_PRODUTO.IDTRATIMPO <> 'V') THEN
            UPDATE IMPITVENDAVM
            SET VRBASECALCVE = VRBASECALCVE + (V_TOTAIMPBCALC * V_TAXABASECALC),
                VRIMPITVENDA = VRIMPITVENDA + ((V_TOTAIMPBCALC * V_TAXABASECALC) * V_TAXAALIQUOTA),
                VRADIMPVE    = VRADIMPVE    + V_ADIMPAF
            WHERE (CDFILIAL     = P_CDFILIAL)         AND
                  (CDPRODUTO    = P_CDPRODUTO)        AND
                  (DTIMPITVENDA = P_DTIMPITVENDA)     AND
                  (CDPRODUTO   = C_PRODUTO.CDPRODUTO) AND
                  (CDIMPOSTO   = C_PRODUTO.CDIMPOSTO);
         END IF;
      END IF;
   END LOOP;

   SELECT SUM(VRIMPITVENDA) INTO P_VRIMPOSTO
   FROM IMPITVENDAVM
   WHERE (CDFILIAL     = P_CDFILIAL)
     AND (CDPRODUTO    = P_CDPRODUTO)
     AND (DTIMPITVENDA = P_DTIMPITVENDA);

   DELETE FROM IMPITVENDAVM
   WHERE (CDFILIAL     = P_CDFILIAL)
     AND (CDPRODUTO    = P_CDPRODUTO)
     AND (DTIMPITVENDA = P_DTIMPITVENDA);
END;
/
